package ru.littlevictor.materialwallpapeer;


import android.app.Activity;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.Gallery;
import android.widget.ImageView;

import java.io.IOException;
import java.util.ArrayList;

public class WallpaperChooser extends Activity
        implements AdapterView.OnItemSelectedListener, View.OnClickListener {

    private Bitmap mBitmap;
    private Gallery mGallery;
    private ImageView mImageView;
    private boolean mIsWallpaperSet;
    private WallpaperLoader mLoader;
    private ArrayList<Integer> mImagesIdArray;
    private ArrayList<Integer> mThumbsIdArray;

    //----------------------------------------------------------------------------------------------
    private void addWallpapers(Resources res, String packageName, int arrayId)
    {
        for (String str : res.getStringArray(arrayId))
        {
            int imageId = res.getIdentifier(str, "drawable", packageName);
            if(imageId != 0)
            {
                int thumbId = res.getIdentifier(str + "_small", "drawable", packageName);
                if (thumbId != 0)
                {
                    mThumbsIdArray.add(thumbId);
                    mImagesIdArray.add(imageId);
                }
            }
        }
    };

    //----------------------------------------------------------------------------------------------
    private void selectWallpaper(int position) {
        if(mIsWallpaperSet) return;
        mIsWallpaperSet = true;
        try {
            setWallpaper(getResources().openRawResource(mImagesIdArray.get(position)));
            setResult(-1);
            finish();
            return;
        }
        catch (IOException localIOException) {

        }
    };

    //----------------------------------------------------------------------------------------------
    @Override
    public void onClick(View paramView) {
        selectWallpaper(mGallery.getSelectedItemPosition());
    };

    //----------------------------------------------------------------------------------------------
    @Override
    public void onCreate(Bundle paramBundle) {
        super.onCreate(paramBundle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        mThumbsIdArray = new ArrayList();
        mImagesIdArray = new ArrayList();
        addWallpapers(getResources(), getPackageName(), R.array.wallpapers);
        setContentView(R.layout.main);
        mGallery = ((Gallery)findViewById(R.id.gallery));
        mGallery.setAdapter(new ImageAdapter(this));
        mGallery.setOnItemSelectedListener(this);
        mGallery.setCallbackDuringFling(false);
        ((Button)findViewById(R.id.set)).setOnClickListener(this);
        mImageView = ((ImageView)findViewById(R.id.wallpaper));
    };

    //----------------------------------------------------------------------------------------------
    @Override
    protected void onDestroy() {
        super.onDestroy();
        if ((mLoader != null) && (this.mLoader.getStatus() != AsyncTask.Status.FINISHED))
        {
            mLoader.cancel(true);
            mLoader = null;
        }
    };

    //----------------------------------------------------------------------------------------------
    @Override
    public void onItemSelected(AdapterView adapterView, View view, int position, long id) {
        if ((this.mLoader != null) && (this.mLoader.getStatus() != AsyncTask.Status.FINISHED)) {
            this.mLoader.cancel();
        }
        mLoader = ((WallpaperLoader)new WallpaperLoader().execute(new Integer[]{position}));
    };

    //----------------------------------------------------------------------------------------------
    @Override
    public void onNothingSelected(AdapterView paramAdapterView) {

    };

    //----------------------------------------------------------------------------------------------
    @Override
    protected void onResume()
    {
        super.onResume();
        this.mIsWallpaperSet = false;
    };

    //----------------------------------------------------------------------------------------------
    private class ImageAdapter extends BaseAdapter {

        private LayoutInflater mLayoutInflater;

        //------------------------------------------------------------------------------------------
        ImageAdapter(WallpaperChooser context)
        {
            mLayoutInflater = context.getLayoutInflater();
        };

        //------------------------------------------------------------------------------------------
        @Override
        public int getCount()
        {
            return mThumbsIdArray.size();
        };

        //------------------------------------------------------------------------------------------
        @Override
        public Object getItem(int paramInt)
        {
            return Integer.valueOf(paramInt);
        };

        //------------------------------------------------------------------------------------------
        @Override
        public long getItemId(int paramInt)
        {
            return paramInt;
        };

        //------------------------------------------------------------------------------------------
        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            ImageView imageview;
            if (convertView == null) {
                imageview = (ImageView)mLayoutInflater.inflate(R.layout.wallpaper_item,
                        parent, false);
            } else {
                imageview = (ImageView)convertView;
            }
            int thumbId = mThumbsIdArray.get(position);
            imageview.setImageResource(thumbId);
            Drawable drawable = imageview.getDrawable();
            if (drawable != null) {
                drawable.setDither(true);
            }
            return imageview;
        };
    };

    //----------------------------------------------------------------------------------------------
    class WallpaperLoader extends AsyncTask<Integer, Void, Bitmap> {

        BitmapFactory.Options mOptions = new BitmapFactory.Options();

        //------------------------------------------------------------------------------------------
        WallpaperLoader()
        {
            mOptions.inDither = false;
            mOptions.inPreferredConfig = Bitmap.Config.ARGB_8888;
        };

        //------------------------------------------------------------------------------------------
        void cancel()
        {
            mOptions.requestCancelDecode();
            super.cancel(true);
        };

        //------------------------------------------------------------------------------------------
        @Override
        protected Bitmap doInBackground(Integer... params)
        {
            if (isCancelled()) return null;
            try
            {
                Bitmap localBitmap = BitmapFactory.decodeResource(getResources(),
                        mImagesIdArray.get(params[0]), mOptions);
                return localBitmap;
            }
            catch (OutOfMemoryError localOutOfMemoryError) {}
            return null;
        };

        //------------------------------------------------------------------------------------------
        @Override
        protected void onPostExecute(Bitmap bitmap) {
            if (bitmap == null) return;
            if (!isCancelled() && !mOptions.mCancel) {
                if (mBitmap != null) mBitmap.recycle();
                Display display = getWindowManager().getDefaultDisplay();
                Point point = new Point();
                display.getSize(point);
                Bitmap bitmap1;
                Drawable drawable;
                if (bitmap.getWidth() > point.x && bitmap.getHeight() > point.y) {
                    bitmap1 = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(),
                            bitmap.getHeight());
                } else if (bitmap.getWidth() < point.x && bitmap.getHeight() > point.y) {
                    int j = point.x - bitmap.getWidth();
                    Bitmap bitmap3 = Bitmap.createScaledBitmap(bitmap, j + bitmap.getWidth(),
                            j + bitmap.getHeight(), true);
                    bitmap1 = Bitmap.createBitmap(bitmap3, 0, 0, bitmap3.getWidth(), point.y);
                } else if (bitmap.getWidth() > point.x && bitmap.getHeight() < point.y) {
                    int i = point.y - bitmap.getHeight();
                    Bitmap bitmap2 = Bitmap.createScaledBitmap(bitmap, i + bitmap.getWidth(),
                            i + bitmap.getHeight(), true);
                    bitmap1 = Bitmap.createBitmap(bitmap2, 0, 0, point.x, bitmap2.getHeight());
                } else {
                    bitmap1 = Bitmap.createScaledBitmap(bitmap, point.x, point.y, true);
                }
                mBitmap = bitmap1;
                mImageView.setImageBitmap(mBitmap);
                drawable = mImageView.getDrawable();
                drawable.setFilterBitmap(true);
                drawable.setDither(true);
                mImageView.postInvalidate();
                mLoader = null;
                return;
            } else {
                bitmap.recycle();
                return;
            }
        };
    };
};
